/*******************************
 * Class for buttons, with debounce
 * Author: Cooper Bills <cbills07@gmail.com>
 * July 2010
 *
 * Uses the internal 20k pullup resistors, so just
 * attach the button to ground and input pin.
 *
 * This class will do the inverting
 *******************************/

#ifndef _BUTTON_H_
#define _BUTTON_H_

#include "common.h" //for basicCallback
 
class Button
{
private:
  unsigned long lastReading;
  byte state;
  byte pin;
  basicCallback pressed;
  basicCallback released;
public:
  Button(byte digitalPin)
  {
    pressed = 0;
    released = 0;
    state = 0;
    pin = digitalPin;
    pinMode(pin, INPUT);    // set pin to input
    digitalWrite(pin, HIGH);  // turn on pullup resistors
    lastReading = 0;
  }
  
  void setPressedCallback(basicCallback f)
  {
    pressed = f;
  }
  
  void setReleasedCallback(basicCallback f)
  {
    released = f;
  }
  
  byte isButtonDown()
  {
    if(state == 2 || state == 3)
      return 1;
    else
      return 0;
  }
  
  void update()
  {
    if(abs(millis() - lastReading) >= 15) //only read every 15ms
    {
      switch(state)
      {
        case 0: //button is released; pin is high
          if(digitalRead(pin) == LOW)
            state = 1;
          break;
        case 1: //button may have been pressed, check again (debounce)
          if(digitalRead(pin) == LOW)
          {
            state = 2;
            if(pressed)
              pressed();
          }
          else
            state = 0;
          break;
        case 2: //button is pressed and confirmed (pin is low), check for possible release
          if(digitalRead(pin) == HIGH)
            state = 3;
          break;
        case 3: //button may have been released, check again (debounce)
          if(digitalRead(pin) == HIGH)
          {
            state = 0;
            if(released)
              released();
          }
          else
            state = 2;
          break;
      }
    }
  }
};

#endif
