/*******************************
 * Class for the pressure sensor.
 * Author: Cooper Bills <cbills07@gmail.com>
 * July 2010
 *
 * Call update() at 40Hz or more for best results
 *******************************/

#ifndef _PRESSURE_H_
#define _PRESSURE_H_

#include "common.h"
 
class PressureSensor
{
private:
  unsigned long lastReading;
  unsigned char sensorPin;
  unsigned int threshold;
  unsigned int value;
  unsigned char aboveThreshold;
  basicCallback thresholdBroken;
  basicCallback dropUnderThreshold;
public:
  PressureSensor(unsigned char analogPin, unsigned char initalThresholdAddition)
  {
    lastReading = 0;
    thresholdBroken = 0;
    aboveThreshold = 0;
    dropUnderThreshold = 0;
    sensorPin = analogPin;
    value = analogRead(sensorPin); //set inital sensor Value
    threshold = analogRead(sensorPin) + initalThresholdAddition; //set inital Threshold
  }
  
  void resetThreshold(unsigned char initalThresholdAddition)
  {
    threshold = analogRead(sensorPin) + initalThresholdAddition; //set inital Threshold
  }
    
  void setRaiseAboveThesholdCallback(basicCallback f)
  {
    thresholdBroken = f;
  }
  
  void setDropBelowThesholdCallback(basicCallback f)
  {
    dropUnderThreshold = f;
  }
  
  void update()
  {
    if(abs(millis() - lastReading) >= 50) //only read every 50ms
    {
      //Serial.println("Sensor Read");
      lastReading = millis();
      value = (value +  analogRead(sensorPin))/2;  //basic smoothing
      
      //Adaptive Threshold:
      if(aboveThreshold) //currently someone on the bed
      {
        //if(threshold == value) threshold--; //adapt if threshold too close
        /*else*/ if(threshold > value) 
        {
          aboveThreshold = 0; //otherwise, if below threshold, the person got up, so trigger callback
          if(thresholdBroken) thresholdBroken();
        }
      }
      else //in grace period
      {
        //if(threshold == value) threshold++; //adapt if threshold too close
        /*else*/ if(threshold < value) 
        {
          aboveThreshold = 1;
          if(dropUnderThreshold) dropUnderThreshold();
        }
      }
    }
  }
  
  unsigned int getValue()
  {
    return value;
  }
 
  unsigned int getThreshold()
  {
    return threshold;
  }
  
  byte isAboveThreshold()
  {
    return aboveThreshold;
  }
};

#endif
