/*******************************
 * Real Time Clock (RTC) Module interface
 * Author: Cooper Bills <cbills07@gmail.com>
 * July 2010
 *
 * void setRTCTime(unsigned char sec, unsigned char min, unsigned char hrs)
 * unsigned char getRTCSeconds() //these three have 25ms delay
 * unsigned char getRTCMinutes()
 * unsigned char getRTCHours()
 *******************************/

#ifndef _RTC_H_ 
#define _RTC_H_

#define RTCLOCKADDR (0xA2>>1) //I2C Address of clock (bit-shifted by 1 to remove r/w bit)

//#include <Wire.h> included in the main file since the arduino IDE doesnt like this...
#include "common.h"

unsigned long rtc_lastUpdate = 99999;
volatile int rtc_hours, rtc_minutes, rtc_seconds, rtc_days, rtc_months, rtc_year;
volatile int rtc_VL = 0;
int rtc_value;

//prototypes
void readRTCTime();
void loop_rtc();

// The setup() method runs once, when the sketch starts
void setup_rtc()   {                
  Wire.begin();
  delay(50);
  loop_rtc(); //call an initial update
}

// should be run as often as possible
void loop_rtc()                     
{  
  if(abs(millis() - rtc_lastUpdate) >= 1000) //update internal time every second
    readRTCTime();
}

unsigned char getRTCSeconds()
{
  return rtc_seconds;
}

unsigned char getRTCMinutes()
{
  return rtc_minutes;
}

unsigned char getRTCHours()
{
  return rtc_hours;
}

unsigned char getRTCVL()
{
  return rtc_VL;
}

void readRTCTime()
{
  char byteRead;
  Wire.beginTransmission(RTCLOCKADDR); //start
  Wire.send(0x02); //set address to 0x02 (VL_SECONDS)
  Wire.endTransmission();
  Wire.requestFrom(RTCLOCKADDR, 7); //request from the read address, 7 bytes total
  delay(5);
  
  if(Wire.available())
  {
    byteRead = Wire.receive();
    rtc_VL = byteRead & 0x80 ? 1 : 0;
    rtc_seconds = fromBCD(byteRead & 0x7F);
  }
  if(Wire.available())
    rtc_minutes = fromBCD(Wire.receive() & 0x7F);
  if(Wire.available())
    rtc_hours = fromBCD(Wire.receive() & 0x3F);
  if(Wire.available())
    rtc_days = fromBCD(Wire.receive() & 0x3F);
  if(Wire.available())
    Wire.receive(); //toss weedays aside for now
  if(Wire.available())
    Wire.receive(); //months
  if(Wire.available())
    Wire.receive(); //Years
  else
    Serial.write("Problem receiving RTC I2C bytes...\n\r");
}
  
void writeRTCByte(unsigned char regAddr, unsigned char val)
{
  //sprintf(stringBuffer, "Writing %d to %d\n\r", val, regAddr);
  //Serial.write(stringBuffer);
  Wire.beginTransmission(RTCLOCKADDR); //start
  Wire.send(regAddr); //set register address
  Wire.send(val);
  if(Wire.endTransmission())
    Serial.write("Problem with RTC I2C Transmission =(\n\r");
  //else
  //  Serial.write("Write Successful!\n\r");
}

void setRTCTime(unsigned char sec, unsigned char min, unsigned char hrs)
{
  writeRTCByte(0x04, toBCD(hrs));
  writeRTCByte(0x03, toBCD(min));
  writeRTCByte(0x02, toBCD(sec));
  readRTCTime();
}

#endif
